@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
@@ -199,11 +199,7 @@ impl SpamFilterConfig {
 impl SpamFilterRules {
     pub fn parse(config: &mut Config) -> SpamFilterRules {
         let mut rules = vec![];
-        for id in config
-            .sub_keys("spam-filter.rule", ".scope")
-            .map(|k| k.to_string())
-            .collect::<Vec<_>>()
-        {
+        for id in config.sub_keys("spam-filter.rule", ".scope") {
             if let Some(rule) = SpamFilterRule::parse(config, id) {
                 rules.push(rule);
             }
@@ -266,11 +262,7 @@ impl SpamFilterRule {
 impl DnsBlConfig {
     pub fn parse(config: &mut Config) -> Self {
         let mut servers = vec![];
-        for id in config
-            .sub_keys("spam-filter.dnsbl.server", ".scope")
-            .map(|k| k.to_string())
-            .collect::<Vec<_>>()
-        {
+        for id in config.sub_keys("spam-filter.dnsbl.server", ".scope") {
             if let Some(server) = DnsBlServer::parse(config, id) {
                 servers.push(server);
             }
