@@ -1,11 +1,14 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::borrow::Cow;
-
+use super::CurrentUserPrincipal;
+use crate::{
+    DavResourceName,
+    common::propfind::{PropFindRequestHandler, SyncTokenUrn},
+};
 use common::{Server, auth::AccessToken};
 use dav_proto::schema::{
     Namespace,
@@ -16,20 +19,14 @@ use dav_proto::schema::{
     request::{DavPropertyValue, PropFind},
     response::{Href, MultiStatus, PropStat, Response},
 };
-use directory::{QueryBy, backend::internal::manage::ManageDirectory};
+use directory::{QueryParams, Type, backend::internal::manage::ManageDirectory};
+use groupware::RFC_3986;
 use groupware::cache::GroupwareCache;
 use hyper::StatusCode;
 use jmap_proto::types::collection::Collection;
-use percent_encoding::NON_ALPHANUMERIC;
+use std::borrow::Cow;
 use trc::AddContext;
 
-use crate::{
-    DavResourceName,
-    common::propfind::{PropFindRequestHandler, SyncTokenUrn},
-};
-
-use super::CurrentUserPrincipal;
-
 pub(crate) trait PrincipalPropFind: Sync + Send {
     fn prepare_principal_propfind_response(
         &self,
@@ -86,7 +83,7 @@ impl PrincipalPropFind for Server {
                 response.set_namespace(Namespace::CardDav);
                 false
             }
-            Collection::Calendar | Collection::CalendarEvent => {
+            Collection::Calendar | Collection::CalendarEvent | Collection::CalendarScheduling => {
                 response.set_namespace(Namespace::CalDav);
                 false
             }
@@ -107,7 +104,7 @@ impl PrincipalPropFind for Server {
             let mut fields = Vec::with_capacity(properties.len());
             let mut fields_not_found = Vec::new();
 
-            let (name, description) = if access_token.primary_id() == account_id {
+            let (name, description, emails, typ) = if access_token.primary_id() == account_id {
                 (
                     Cow::Borrowed(access_token.name.as_str()),
                     access_token
@@ -115,21 +112,30 @@ impl PrincipalPropFind for Server {
                         .as_deref()
                         .unwrap_or(&access_token.name)
                         .to_string(),
+                    Cow::Borrowed(access_token.emails.as_slice()),
+                    Type::Individual,
                 )
             } else {
                 self.directory()
-                    .query(QueryBy::Id(account_id), false)
+                    .query(QueryParams::id(account_id).with_return_member_of(false))
                     .await
                     .caused_by(trc::location!())?
                     .map(|p| {
                         let name = p.name;
                         let description = p.description.unwrap_or_else(|| name.clone());
-                        (Cow::Owned(name.to_string()), description.to_string())
+                        (
+                            Cow::Owned(name.to_string()),
+                            description.to_string(),
+                            Cow::Owned(p.emails),
+                            p.typ,
+                        )
                     })
                     .unwrap_or_else(|| {
                         (
                             Cow::Owned(format!("_{}", account_id)),
                             format!("_{}", account_id),
+                            Cow::Owned(vec![]),
+                            Type::Individual,
                         )
                     })
             };
@@ -212,7 +218,7 @@ impl PrincipalPropFind for Server {
                                 vec![Href(format!(
                                     "{}/{}/",
                                     DavResourceName::Principal.base_path(),
-                                    percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                                    percent_encoding::utf8_percent_encode(&name, RFC_3986),
                                 ))],
                             ));
                         }
@@ -257,35 +263,78 @@ impl PrincipalPropFind for Server {
                                 vec![Href(format!(
                                     "{}/{}/",
                                     DavResourceName::Principal.base_path(),
-                                    percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                                    percent_encoding::utf8_percent_encode(&name, RFC_3986),
                                 ))],
                             ));
                         }
                         PrincipalProperty::CalendarHomeSet => {
+                            let hrefs =
+                                build_home_set(self, access_token, name.as_ref(), account_id, true)
+                                    .await
+                                    .caused_by(trc::location!())?;
+
+                            fields.push(DavPropertyValue::new(property.clone(), hrefs));
+                            response.set_namespace(Namespace::CalDav);
+                        }
+                        PrincipalProperty::AddressbookHomeSet => {
+                            let hrefs = build_home_set(
+                                self,
+                                access_token,
+                                name.as_ref(),
+                                account_id,
+                                false,
+                            )
+                            .await
+                            .caused_by(trc::location!())?;
+
+                            fields.push(DavPropertyValue::new(property.clone(), hrefs));
+                            response.set_namespace(Namespace::CardDav);
+                        }
+
+                        PrincipalProperty::PrincipalAddress => {
+                            fields_not_found.push(DavPropertyValue::empty(property.clone()));
+                            response.set_namespace(Namespace::CardDav);
+                        }
+                        PrincipalProperty::CalendarUserAddressSet => {
+                            fields.push(DavPropertyValue::new(
+                                property.clone(),
+                                emails
+                                    .iter()
+                                    .filter(|email| !email.starts_with("@"))
+                                    .take(1)
+                                    .map(|email| Href(format!("mailto:{email}",)))
+                                    .collect::<Vec<_>>(),
+                            ));
+                            response.set_namespace(Namespace::CalDav);
+                        }
+                        PrincipalProperty::CalendarUserType => {
+                            fields.push(DavPropertyValue::new(
+                                property.clone(),
+                                typ.as_str().to_uppercase(),
+                            ));
+                            response.set_namespace(Namespace::CalDav);
+                        }
+                        PrincipalProperty::ScheduleInboxURL => {
                             fields.push(DavPropertyValue::new(
                                 property.clone(),
                                 vec![Href(format!(
-                                    "{}/{}/",
-                                    DavResourceName::Cal.base_path(),
-                                    percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                                    "{}/{}/inbox/",
+                                    DavResourceName::Scheduling.base_path(),
+                                    percent_encoding::utf8_percent_encode(&name, RFC_3986),
                                 ))],
                             ));
                             response.set_namespace(Namespace::CalDav);
                         }
-                        PrincipalProperty::AddressbookHomeSet => {
+                        PrincipalProperty::ScheduleOutboxURL => {
                             fields.push(DavPropertyValue::new(
                                 property.clone(),
                                 vec![Href(format!(
-                                    "{}/{}/",
-                                    DavResourceName::Card.base_path(),
-                                    percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                                    "{}/{}/outbox/",
+                                    DavResourceName::Scheduling.base_path(),
+                                    percent_encoding::utf8_percent_encode(&name, RFC_3986),
                                 ))],
                             ));
-                            response.set_namespace(Namespace::CardDav);
-                        }
-                        PrincipalProperty::PrincipalAddress => {
-                            fields_not_found.push(DavPropertyValue::empty(property.clone()));
-                            response.set_namespace(Namespace::CardDav);
+                            response.set_namespace(Namespace::CalDav);
                         }
                     },
                     _ => {
@@ -310,7 +359,7 @@ impl PrincipalPropFind for Server {
                 Href(format!(
                     "{}/{}/",
                     base_path,
-                    percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                    percent_encoding::utf8_percent_encode(&name, RFC_3986),
                 )),
                 prop_stats,
             ));
@@ -351,12 +400,54 @@ impl PrincipalPropFind for Server {
             Ok(Href(format!(
                 "{}/{}/",
                 DavResourceName::Principal.base_path(),
-                percent_encoding::utf8_percent_encode(&name, NON_ALPHANUMERIC),
+                percent_encoding::utf8_percent_encode(&name, RFC_3986),
             )))
         }
     }
 }
 
+pub(crate) async fn build_home_set(
+    server: &Server,
+    access_token: &AccessToken,
+    name: &str,
+    account_id: u32,
+    is_calendar: bool,
+) -> trc::Result<Vec<Href>> {
+    let (collection, resource_name) = if is_calendar {
+        (Collection::Calendar, DavResourceName::Cal)
+    } else {
+        (Collection::AddressBook, DavResourceName::Card)
+    };
+
+    let mut hrefs = Vec::new();
+    hrefs.push(Href(format!(
+        "{}/{}/",
+        resource_name.base_path(),
+        percent_encoding::utf8_percent_encode(name, RFC_3986),
+    )));
+
+    if account_id == access_token.primary_id() {
+        for account_id in access_token.all_ids_by_collection(collection) {
+            if account_id != access_token.primary_id() {
+                let other_name = server
+                    .store()
+                    .get_principal_name(account_id)
+                    .await
+                    .caused_by(trc::location!())?
+                    .unwrap_or_else(|| format!("_{account_id}"));
+
+                hrefs.push(Href(format!(
+                    "{}/{}/",
+                    resource_name.base_path(),
+                    percent_encoding::utf8_percent_encode(&other_name, RFC_3986),
+                )));
+            }
+        }
+    }
+
+    Ok(hrefs)
+}
+
 fn all_props(collection: Collection, all_props: Option<&[DavProperty]>) -> Vec<DavProperty> {
     if collection == Collection::Principal {
         vec![
