@@ -1,11 +1,14 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{collections::hash_map::Entry, fmt, str::FromStr};
-
+use crate::{
+    ArchivedPrincipal, FALLBACK_ADMIN_ID, Permission, PermissionGrant, Principal, PrincipalData,
+    ROLE_ADMIN, Type,
+    backend::internal::{PrincipalField, PrincipalSet, PrincipalUpdate, PrincipalValue},
+};
 use ahash::AHashSet;
 use nlp::tokenizers::word::WordTokenizer;
 use serde::{
@@ -13,17 +16,13 @@ use serde::{
     de::{self, IgnoredAny, Visitor},
     ser::SerializeMap,
 };
+use std::{collections::hash_map::Entry, fmt, str::FromStr};
 use store::{
     U64_LEN,
     backend::MAX_TOKEN_LENGTH,
     write::{BatchBuilder, DirectoryClass},
 };
 
-use crate::{
-    ArchivedPrincipal, Permission, PermissionGrant, Principal, PrincipalData, ROLE_ADMIN, Type,
-    backend::internal::{PrincipalField, PrincipalSet, PrincipalUpdate, PrincipalValue},
-};
-
 impl Principal {
     pub fn new(id: u32, typ: Type) -> Self {
         Self {
@@ -73,13 +72,20 @@ impl Principal {
     }
 
     // SPDX-SnippetBegin
-    // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+    // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
     // SPDX-License-Identifier: LicenseRef-SEL
+
+    #[cfg(feature = "enterprise")]
     pub fn tenant(&self) -> Option<u32> {
         self.tenant
     }
     // SPDX-SnippetEnd
 
+    #[cfg(not(feature = "enterprise"))]
+    pub fn tenant(&self) -> Option<u32> {
+        None
+    }
+
     pub fn description(&self) -> Option<&str> {
         self.description.as_deref()
     }
@@ -306,7 +312,7 @@ impl Principal {
 
     pub fn fallback_admin(fallback_pass: impl Into<String>) -> Self {
         Principal {
-            id: u32::MAX,
+            id: FALLBACK_ADMIN_ID,
             typ: Type::Individual,
             name: "Fallback Administrator".into(),
             secrets: vec![fallback_pass.into()],
@@ -349,13 +355,19 @@ impl PrincipalSet {
     }
 
     // SPDX-SnippetBegin
-    // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+    // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
     // SPDX-License-Identifier: LicenseRef-SEL
+    #[cfg(feature = "enterprise")]
     pub fn tenant(&self) -> Option<u32> {
         self.get_int(PrincipalField::Tenant).map(|v| v as u32)
     }
     // SPDX-SnippetEnd
 
+    #[cfg(not(feature = "enterprise"))]
+    pub fn tenant(&self) -> Option<u32> {
+        None
+    }
+
     pub fn description(&self) -> Option<&str> {
         self.get_str(PrincipalField::Description)
     }
@@ -912,7 +924,7 @@ impl Type {
             1 => Type::Group,
             2 => Type::Resource,
             3 => Type::Location,
-            4 => Type::Individual, // legacy
+            4 => Type::Other, // legacy
             5 => Type::List,
             6 => Type::Other,
             7 => Type::Domain,
@@ -1095,7 +1107,8 @@ impl<'de> serde::Deserialize<'de> for PrincipalSet {
                         }
                         PrincipalField::Description
                         | PrincipalField::Tenant
-                        | PrincipalField::Picture => {
+                        | PrincipalField::Picture
+                        | PrincipalField::Locale => {
                             if let Some(v) = map.next_value::<Option<String>>()? {
                                 if v.len() <= MAX_STRING_LEN {
                                     PrincipalValue::String(v)
@@ -1407,13 +1420,22 @@ impl Permission {
                 | Permission::DavCalQuery
                 | Permission::DavCalMultiGet
                 | Permission::DavCalFreeBusyQuery
+                | Permission::CalendarAlarms
+                | Permission::CalendarSchedulingSend
+                | Permission::CalendarSchedulingReceive
         )
     }
 
+    #[cfg(not(feature = "enterprise"))]
+    pub const fn is_tenant_admin_permission(&self) -> bool {
+        false
+    }
+
     // SPDX-SnippetBegin
-    // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+    // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
     // SPDX-License-Identifier: LicenseRef-SEL
 
+    #[cfg(feature = "enterprise")]
     pub const fn is_tenant_admin_permission(&self) -> bool {
         matches!(
             self,
