@@ -1,11 +1,24 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{net::IpAddr, sync::Arc};
-
+use crate::{
+    HttpSessionManager,
+    auth::{
+        authenticate::{Authenticator, HttpHeaders},
+        oauth::{
+            FormData, auth::OAuthApiHandler, openid::OpenIdHandler,
+            registration::ClientRegistrationHandler, token::TokenHandler,
+        },
+    },
+    autoconfig::Autoconfig,
+    form::FormHandler,
+    management::{
+        ManagementApi, ToManageHttpResponse, UnauthorizedResponse, troubleshoot::TroubleshootApi,
+    },
+};
 use common::{
     Inner, KV_ACME, Server,
     auth::{AccessToken, oauth::GrantType},
@@ -16,10 +29,10 @@ use common::{
 };
 use dav::{DavMethod, request::DavRequestHandler};
 use directory::Permission;
-use groupware::DavResourceName;
+use groupware::{DavResourceName, calendar::itip::ItipIngest};
 use http_proto::{
-    DownloadResponse, HttpContext, HttpRequest, HttpResponse, HttpResponseBody, HttpSessionData,
-    JsonProblemResponse, ToHttpResponse, form_urlencoded, request::fetch_body,
+    DownloadResponse, HtmlResponse, HttpContext, HttpRequest, HttpResponse, HttpResponseBody,
+    HttpSessionData, JsonProblemResponse, ToHttpResponse, form_urlencoded, request::fetch_body,
 };
 use hyper::{
     Method, StatusCode, body,
@@ -37,27 +50,14 @@ use jmap::{
     websocket::upgrade::WebSocketUpgrade,
 };
 use jmap_proto::{
-    request::Request,
+    request::{Request, capability::Session},
     types::{blob::BlobId, id::Id},
 };
+use std::{net::IpAddr, sync::Arc};
 use store::dispatch::lookup::KeyValue;
 use trc::SecurityEvent;
 use utils::url_params::UrlParams;
 
-use crate::{
-    HttpSessionManager,
-    auth::{
-        authenticate::{Authenticator, HttpHeaders},
-        oauth::{
-            FormData, auth::OAuthApiHandler, openid::OpenIdHandler,
-            registration::ClientRegistrationHandler, token::TokenHandler,
-        },
-    },
-    autoconfig::Autoconfig,
-    form::FormHandler,
-    management::{ManagementApi, ToManageHttpResponse, troubleshoot::TroubleshootApi},
-};
-
 pub trait ParseHttp: Sync + Send {
     fn parse_http_request(
         &self,
@@ -199,6 +199,26 @@ impl ParseHttp for Server {
                             .upgrade_websocket_connection(req, access_token, session)
                             .await;
                     }
+                    ("session", &Method::GET) => {
+                        return if req.headers().contains_key(header::AUTHORIZATION) {
+                            // Authenticate request
+                            let (_in_flight, access_token) =
+                                self.authenticate_headers(&req, &session, false).await?;
+
+                            self.handle_session_resource(
+                                ctx.resolve_response_url(self).await,
+                                access_token,
+                            )
+                            .await
+                            .map(|s| s.into_http_response())
+                        } else {
+                            Ok(Session::new(
+                                ctx.resolve_response_url(self).await,
+                                &self.core.jmap.capabilities,
+                            )
+                            .into_http_response())
+                        };
+                    }
                     (_, &Method::OPTIONS) => {
                         return Ok(JsonProblemResponse(StatusCode::NO_CONTENT).into_http_response());
                     }
@@ -215,7 +235,7 @@ impl ParseHttp for Server {
                             "DAV",
                             concat!(
                                 "1, 2, 3, access-control, extended-mkcol, calendar-access, ",
-                                "calendar-no-timezone, addressbook"
+                                "calendar-auto-schedule, calendar-no-timezone, addressbook"
                             ),
                         )
                         .with_header(
@@ -241,14 +261,9 @@ impl ParseHttp for Server {
             }
             ".well-known" => match (path.next().unwrap_or_default(), req.method()) {
                 ("jmap", &Method::GET) => {
-                    // Authenticate request
-                    let (_in_flight, access_token) =
-                        self.authenticate_headers(&req, &session, false).await?;
-
-                    return self
-                        .handle_session_resource(ctx.resolve_response_url(self).await, access_token)
-                        .await
-                        .map(|s| s.into_http_response());
+                    return Ok(HttpResponse::new(StatusCode::TEMPORARY_REDIRECT)
+                        .with_no_cache()
+                        .with_location("/jmap/session"));
                 }
                 ("caldav", _) => {
                     return Ok(HttpResponse::new(StatusCode::TEMPORARY_REDIRECT)
@@ -394,7 +409,7 @@ impl ParseHttp for Server {
                                 params.get("token"),
                             ) {
                                 // SPDX-SnippetBegin
-                                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                                // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
                                 // SPDX-License-Identifier: LicenseRef-SEL
                                 #[cfg(feature = "enterprise")]
                                 (Some("telemetry"), Some("traces"), Some(token))
@@ -412,14 +427,14 @@ impl ParseHttp for Server {
                                 (Some("troubleshoot"), _, Some(token)) => {
                                     (GrantType::Troubleshoot, token)
                                 }
-                                _ => return Err(err),
+                                _ => return Ok(HttpResponse::unauthorized(false)),
                             };
                             let token_info =
                                 self.validate_access_token(grant_type.into(), token).await?;
 
                             return match grant_type {
                                 // SPDX-SnippetBegin
-                                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                                // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
                                 // SPDX-License-Identifier: LicenseRef-SEL
                                 #[cfg(feature = "enterprise")]
                                 GrantType::LiveTracing | GrantType::LiveMetrics => {
@@ -463,6 +478,35 @@ impl ParseHttp for Server {
                     return self.handle_autoconfig_request(&req).await;
                 }
             }
+            "calendar" => {
+                // Limit anonymous requests
+                self.is_http_anonymous_request_allowed(&session.remote_ip)
+                    .await?;
+
+                if self.core.groupware.itip_http_rsvp_url.is_some()
+                    && req.method() == Method::GET
+                    && path.next().unwrap_or_default() == "rsvp"
+                {
+                    return self
+                        .http_rsvp_handle(
+                            req.uri().query().unwrap_or_default(),
+                            req.headers()
+                                .get(header::ACCEPT_LANGUAGE)
+                                .and_then(|v| v.to_str().ok())
+                                .map(|lang| {
+                                    let lang = lang.split_once(',').map_or(lang, |(l, _)| l);
+                                    lang.split_once(';').map_or(lang, |(l, _)| l)
+                                })
+                                .unwrap_or("en"),
+                        )
+                        .await
+                        .map(|response| {
+                            HtmlResponse::new(response)
+                                .into_http_response()
+                                .with_no_store()
+                        });
+                }
+            }
             "autodiscover" => {
                 if req.method() == Method::POST
                     && path.next().unwrap_or_default() == "autodiscover.xml"
@@ -537,12 +581,11 @@ impl ParseHttp for Server {
                 }
                 _ => (),
             },
+            // SPDX-SnippetBegin
+            // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
+            // SPDX-License-Identifier: LicenseRef-SEL
             #[cfg(feature = "enterprise")]
             "logo.svg" if self.is_enterprise_edition() => {
-                // SPDX-SnippetBegin
-                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
-                // SPDX-License-Identifier: LicenseRef-SEL
-
                 match self
                     .logo_resource(
                         req.headers()
@@ -567,9 +610,8 @@ impl ParseHttp for Server {
                 if !resource.is_empty() {
                     return Ok(resource.into_http_response());
                 }
-
-                // SPDX-SnippetEnd
             }
+            // SPDX-SnippetEnd
             "form" => {
                 if let Some(form) = &self.core.network.contact_form {
                     match *req.method() {
