@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
@@ -18,6 +18,7 @@ use jmap_proto::{
 };
 use mail_parser::MessageParser;
 use std::future::Future;
+use trc::AddContext;
 use utils::map::vec_map::VecMap;
 
 pub trait EmailImport: Sync + Send {
@@ -46,8 +47,15 @@ impl EmailImport for Server {
             None
         };
 
-        // Obtain quota
-        let resource_token = self.get_resource_token(access_token, account_id).await?;
+        // Obtain import access token
+        let import_access_token = if account_id != access_token.primary_id() {
+            self.get_access_token(account_id)
+                .await
+                .caused_by(trc::location!())?
+                .into()
+        } else {
+            None
+        };
 
         let mut response = ImportEmailResponse {
             account_id: request.account_id,
@@ -117,7 +125,7 @@ impl EmailImport for Server {
                 .email_ingest(IngestEmail {
                     raw_message: &raw_message,
                     message: MessageParser::new().parse(&raw_message),
-                    resource: resource_token.clone(),
+                    access_token: import_access_token.as_deref().unwrap_or(access_token),
                     mailbox_ids,
                     keywords: email.keywords,
                     received_at: email.received_at.map(|r| r.into()),
