@@ -84,6 +84,16 @@ impl EmailSet for Server {
                 (None, None, None)
             };
 
+        // Obtain import access token
+        let import_access_token = if account_id != access_token.primary_id() {
+            self.get_access_token(account_id)
+                .await
+                .caused_by(trc::location!())?
+                .into()
+        } else {
+            None
+        };
+
         let mut last_change_id = None;
         let will_destroy = request.unwrap_destroy();
 
@@ -703,7 +713,7 @@ impl EmailSet for Server {
                 .email_ingest(IngestEmail {
                     raw_message: &raw_message,
                     message: MessageParser::new().parse(&raw_message),
-                    access_token,
+                    access_token: import_access_token.as_deref().unwrap_or(access_token),
                     mailbox_ids: mailboxes,
                     keywords,
                     received_at,
