@@ -1,17 +1,16 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{collections::HashMap, sync::Arc};
-
+use crate::blob::download::BlobDownload;
 use common::{
     Server,
+    config::smtp::queue::QueueName,
     listener::{ServerInstance, stream::NullIo},
     storage::index::ObjectIndexBuilder,
 };
-
 use email::{
     identity::Identity,
     message::metadata::MessageMetadata,
@@ -41,13 +40,12 @@ use smtp::{
     queue::spool::SmtpSpool,
 };
 use smtp_proto::{MailFrom, RcptTo, request::parser::Rfc5321Parser};
+use std::future::Future;
+use std::{collections::HashMap, sync::Arc, time::Duration};
 use store::write::{BatchBuilder, now};
 use trc::AddContext;
 use utils::{BlobHash, map::vec_map::VecMap, sanitize_email};
 
-use crate::blob::download::BlobDownload;
-use std::future::Future;
-
 pub trait EmailSubmissionSet: Sync + Send {
     fn email_submission_set(
         &self,
@@ -166,10 +164,11 @@ impl EmailSubmissionSet for Server {
 
             match undo_status {
                 Some(undo_status) if undo_status == "canceled" => {
-                    if let Some(queue_message) = self.read_message(queue_id).await {
+                    if let Some(queue_message) =
+                        self.read_message(queue_id, QueueName::default()).await
+                    {
                         // Delete message from queue
-                        let message_due = queue_message.next_event().unwrap_or_default();
-                        queue_message.remove(self, message_due).await;
+                        queue_message.remove(self, None).await;
 
                         // Update record
                         let mut new_submission = submission.inner.clone();
@@ -604,6 +603,7 @@ impl EmailSubmissionSet for Server {
             // RCPT TO
             let mut responses = Vec::new();
             let mut has_success = false;
+            session.params.rcpt_errors_wait = Duration::from_secs(0);
             for rcpt in rcpt_to {
                 let addr = rcpt.address.clone();
                 let _ = session.handle_rcpt_to(rcpt).await;
